#include <stdio.h>
#include <iostream>
#include <algorithm>
#include <vector>
#include <string>


#define SQR(X) ((X)*(X))


using namespace std;


typedef vector<int> VI;
typedef vector<VI> VVI;


const int MaxN = 50000;
const int MaxW = 2560;
const int MaxH = 2048;
const int MaxL = 25;


class Point {
	friend istream& operator >> (istream& in, Point& p) {
		in >> p.x >> p.y;
		p.x--; p.y--;
		return in;
	}


private:
	int x, y;


public:
	Point(int _x = 0, int _y = 0): x(_x), y(_y) { };

	
	bool in_range(int MinX, int MinY, int MaxX, int MaxY) const {
		return MinX-1 <= x && x <= MaxX-1 && MinY-1 <= y && y <= MaxY-1;
	}


	int get_x() const {
		return x;
	}


	int get_y() const {
		return y;
	}

};


class Picture {
private:
	static const int empty = 0;
	static const int point = 1;
	static const int bound = 2;
	int w, h, l;
	VVI pic;


public:	
	Picture(int _w, int _h, int _l): w(_w), h(_h), l(_l) {
		pic.resize(h);
		for (VVI::iterator it = pic.begin(); it < pic.end(); it++) {
			it->resize(w);
		}
	}

	
	bool add_point(const Point& p) {
		bool result = pic[p.get_y()][p.get_x()] == empty;
		if (result) {
			int cx = p.get_x();
			int cy = p.get_y();
			for (int i = cy - l; i <= cy + l; i++) {
				for (int j = cx - l; j <= cx + l; j++) {
					if (i < 0 || j < 0 || i >= h || j >= w) {
						continue;
					}
					if (SQR(cy - i) + SQR(cx - j) <= SQR(l)) {
						pic[i][j] = bound;
					}
				}
			}
			pic[cy][cx] = point;
		}
		return result;
	}

};


Point point;
int n, w, h, l;
VI ans;


inline void my_assert(bool cond, const string& msg) {
	if (!cond) {
		cerr << "WRONG INPUT : ";
		cerr << msg << endl;		
		throw msg;
	}
}


int main() {
	freopen("input.txt", "r", stdin);
	freopen("output.txt", "w", stdout);
	cin >> n >> w >> h >> l;
	my_assert(0 < n && n <= MaxN, "N is out of range");
	my_assert(0 <= l && l <= MaxL, "L is out of range");
	my_assert(0 < w && w <= MaxW, "W is out of range");
	my_assert(0 < h && h <= MaxH, "H is out of range");
	Picture graph(w, h, l);
	for (int i = 0; i < n; i++) {
		cin >> point;
		my_assert(point.in_range(1, 1, w, h), "one of Points is out of range");
		if (graph.add_point(point)) {
			ans.push_back(i+1);
		}
	}
	cout << ans.size() << endl;
	for (VI::const_iterator it = ans.begin(); it < ans.end(); it++) {
		cout << (*it) << endl;
	}
	fclose(stdin);
	fclose(stdout);
	return 0;
}
